// Copyright 2020 The Wire Authors
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
//+build wireinject

package main

import (
	"github.com/google/wire"
)

/* blockComment returns Foo and has a /*- style doc comment */
func blockComment() *Foo {
	panic(wire.Build(wire.Struct(new(Foo))))
}

// lineComment returns Bar and has a //- style doc comment
func lineComment() *Bar {
	panic(wire.Build(wire.Struct(new(Bar))))
}
