// Copyright 2018 The Wire Authors
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package main

import (
	"io"
	"strings"

	"github.com/google/wire"
)

type context struct{}

func main() {}

type Foo string
type Bar io.Reader

var Set = wire.NewSet(provideFoo)
var SuperSet = wire.NewSet(Set)
var SetWithDuplicateBindings = wire.NewSet(Set, SuperSet)

func provideFoo() Foo {
	return Foo("foo")
}

func provideFooAgain() Foo {
	return Foo("foo foo")
}

func provideBar() Bar {
	return io.Reader(strings.NewReader("hello"))
}
