// Copyright 2019 The Wire Authors
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package bar

import (
	"example.com/foo"
)

type Config struct {
	V int
}

type Service struct {
	Cfg *Config
	F   *foo.Service
}

func New(cfg *Config, f *foo.Service) *Service {
	return &Service{Cfg: cfg, F: f}
}
