// Code generated by Wire. DO NOT EDIT.

//go:generate go run -mod=mod github.com/google/wire/cmd/wire
//go:build !wireinject
// +build !wireinject

package main

import (
	"io"
	"os"
)

// Injectors from wire.go:

func inject() io.Writer {
	file := _wireFileValue
	return file
}

var (
	_wireFileValue = os.Stdout
)
