// Copyright 2018 The Wire Authors
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package main

import (
	"fmt"
)

func main() {
	bar := injectBar(func() *Foo { return &Foo{42} })
	fmt.Println(bar.Name)
}

type Foo struct {
	Val int
}

type Bar struct {
	Name string
}

func NewBar(f *Foo) *Bar {
	return &Bar{Name: fmt.Sprintf("foo value %d", f.Val)}
}
